package com.doc.biz.service.impl;

import com.doc.biz.domain.DocInfo;
import com.doc.biz.domain.EsDocInfo;
import com.doc.biz.service.IElasticSearchService;
import com.doc.biz.service.IEsDocInfoService;
import com.doc.biz.service.IMongoService;
import com.doc.biz.service.IOcrService;
import com.doc.biz.vo.DocumentVO;
import com.doc.common.config.EsConfig;
import com.doc.common.constant.Constants;
import com.doc.common.utils.FileContentUtils;
import com.doc.common.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;

/**
 * ElasticSearch 索引接口
 *
 * @author wukai
 * @date 2023-08-15
 */
@Service
@Slf4j
public class ElasticSearchServiceImpl implements IElasticSearchService {
    @Resource
    private IEsDocInfoService esDocInfoService;
    @Resource
    private EsConfig esConfig;
    @Resource
    private IMongoService mongoService;
    @Resource
    private IOcrService ocrService;

    /**
     * 文件内容入es库
     *
     * @param info 文档信息
     */
    @Override
    @Async("threadPoolTaskExecutor")
    public void save(DocInfo info) {
        if (info.getFileSize() == 0) {
            return;
        }
        String fileType = info.getFileType().toLowerCase();
        Map<String, Function<byte[], String>> handlerMap = createHandlerMap(fileType);
        String content = getContent(handlerMap, fileType, info.getFileId());
        if (StringUtils.isNotEmpty(content)) {
            comboIndex(info.getSpaceId());
            try {
                EsDocInfo esDocInfo = new EsDocInfo(info.getDocId(), content);
                esDocInfoService.save(esDocInfo);
            } catch (Exception e) {
                log.error("解析文件错误？？？{}", e.getMessage());
            }
        }
    }

    /**
     * 通过ID删除ES内容
     *
     * @param info 文档信息
     */
    @Override
    public void delete(DocInfo info) {
        comboIndex(info.getSpaceId());
        esDocInfoService.deleteById(info.getDocId());
    }

    /**
     * 组装ES索引名
     *
     * @param spaceId 空间ID
     */
    private void comboIndex(Long spaceId) {
        String indexName = "docs_" + spaceId;
        esConfig.setIndexName(indexName);
    }

    /**
     * 创建文件访问类型MAP
     *
     * @param fileType 文件类型
     * @return map
     */
    private Map<String, Function<byte[], String>> createHandlerMap(String fileType) {
        Map<String, Function<byte[], String>> handlerMap = new HashMap<>(16);
        switch (fileType) {
            case ".docx":
            case ".doc":
            case ".wps":
                //word wps文档
                handlerMap.put(fileType, FileContentUtils::getContentDocx);
                break;
            case ".xls":
            case ".et":
                handlerMap.put(fileType, FileContentUtils::getContentXls);
                break;
            case ".xlsx":
                handlerMap.put(fileType, FileContentUtils::getContentXlsx);
                break;
            case ".ppt":
            case ".dps":
                handlerMap.put(fileType, FileContentUtils::getContentPpt);
                break;
            case ".pptx":
                handlerMap.put(fileType, FileContentUtils::getContentPptx);
                break;
            case ".pdf":
                handlerMap.put(fileType, FileContentUtils::getContentPdf);
                break;
            default:
                break;
        }
        return handlerMap;
    }

    /**
     * 获取文件内容
     *
     * @param handlerMap map
     * @param fileType   文件类型
     * @param fileId     文件ID
     * @return 解析结果
     */
    private String getContent(Map<String, Function<byte[], String>> handlerMap, String fileType, String fileId) {
        Function<byte[], String> handler = handlerMap.get(fileType);
        if (Constants.IMAGE_EXTENSION.contains(fileType)) {
            try {
                return ocrService.recognition(fileId);
            } catch (Exception e) {
                log.error("解析图片错误：{}", e.getMessage());
            }
        } else if (handler != null) {
            DocumentVO vo = mongoService.downloadFile(fileId);
            if (vo != null) {
                return handler.apply(vo.getData());
            }
        }
        return "";
    }

}
